<?php

	Class extension_edit_plus extends Extension{

		public function install(){
			Symphony::Configuration()->set('stylesheets_path', 'css','edit_plus');
			Symphony::Configuration()->set('scripts_path', 'js','edit_plus');
		}

		public function getSubscribedDelegates(){
			return array(
				array(
					'page' => '/system/preferences/',
					'delegate' => 'AddCustomPreferenceFieldsets',
					'callback' => 'appendPreferences'
				),
				array(
					'page' => '/backend/',
					'delegate' => 'ExtensionsAddToNavigation',
					'callback' => 'addToNavigation'
				),
				array(
					'page' => '/backend/',
					'delegate' => 'InitaliseAdminPageHead',
					'callback' => 'addFilesToHead'
				),
				array(
					'page' => '/backend/',
					'delegate' => 'AdminPagePostCallback',
					'callback' => 'adminPagePostCallback'
				)
			);
		}

		public function appendPreferences(&$context){
			$fieldset = new XMLElement('fieldset');
			$fieldset->setAttribute('class', 'settings');
			$fieldset->appendChild(new XMLElement('legend', 'Edit Plus'));

			$div = new XMLElement('div');
			$div->setAttribute('class', 'two columns');
			$label = Widget::Label(__('Stylesheets Folder Path'));
			$label->setAttribute('class', 'column');
			$label->appendChild(
				Widget::Input('settings[edit_plus][stylesheets_path]', Symphony::Configuration()->get('stylesheets_path', 'edit_plus'))
			);
			$div->appendChild($label);

			$label = Widget::Label(__('Scripts Folder Path'));
			$label->setAttribute('class', 'column');
			$label->appendChild(
				Widget::Input('settings[edit_plus][scripts_path]', Symphony::Configuration()->get('scripts_path', 'edit_plus'))
			);
			$div->appendChild($label);

			$fieldset->appendChild($div);
			$fieldset->appendChild(new XMLElement('p', __('Path is relative to the <code>workspace</code> folder.'), array('class' => 'help')));
			$context['wrapper']->appendChild($fieldset);
		}

		public function save($context){
		}

		public function addFilesToHead(){
			$uri = $_SERVER['REQUEST_URI'];
			$editing_assets = preg_match('/assets\/(stylesheets|scripts)\/(new|edit)/', $uri);
			if ($editing_assets or preg_match('/blueprints\/utilities\/(new|edit)/', $uri) or strpos($uri, 'blueprints/pages/template')){
				Administration::instance()->Page->addStylesheetToHead(URL . '/extensions/edit_plus/assets/edit-plus.css', 'screen');
				Administration::instance()->Page->addScriptToHead(URL . '/extensions/edit_plus/assets/edit-plus.js');
				if(!$editing_assets){
					Administration::instance()->Page->addStylesheetToHead(URL . '/extensions/edit_plus/assets/xslt-highlighter.css', 'screen');
					Administration::instance()->Page->addScriptToHead(URL . '/extensions/edit_plus/assets/xslt-highlighter.js');
				}
			}
		}

		public function addToNavigation(&$context){
			//$navigation = $context['navigation'];
			$context['navigation'][250] = array(
				'name' => 'Assets',
				'type' => 'structure',
				'index' => '250',
				'children' => array(
					array(
						'link' => '/assets/stylesheets/',
						'name' => 'Stylesheets',
						'visible' => 'yes'
					),
					array(
						'link' => '/assets/stylesheets/new/',
						'name' => 'Stylesheets New',
						'visible' => 'no'
					),
					array(
						'link' => '/assets/stylesheets/edit/',
						'name' => 'Stylesheets Edit',
						'visible' => 'no'
					),
					array(
						'link' => '/assets/scripts/',
						'name' => 'Scripts',
						'visible' => 'yes'
					)
				)
			);
			//$context['navigation'] = $navigation;
		}

		public function adminPagePostCallback(&$context) {
			$parts = $context['parts'];
			if($parts[0] == 'assets' and isset($parts[1])) {
				if($parts[1] == 'stylesheets' or $parts[1] == 'scripts') {
					$callback = $context['callback'];
					$callback['driver'] = 'assets';
					$callback['driver_location'] = EXTENSIONS . '/edit_plus/content/content.assets.php';
					$callback['pageroot'] = '/extensions/edit_plus/content/';
					$callback['classname'] = 'contentExtensionEdit_plusAssets';
					if(isset($parts[2])) {
						$callback['context'] = explode('/', $parts[2]);
					}
					$context['callback'] = $callback;
				}
			}
		}
	}
?>